import 'dart:io';
import 'dart:convert';

import 'package:http/http.dart' as http;
import '../src/exception/ApiException.dart';
import '../src/support/Request.dart';
import '../src/support/Response.dart';

///
/// API Client
///
class ApiClient {
  String _url;

  ///
  /// Constructor
  ///
  ApiClient(this._url);

  ///
  /// request api
  ///
  /// [command] api command
  /// [parameters] api parameters
  ///
  Future<Response> request(String command, {Map<String, dynamic> parameters = const {}}) async {
    var headers = {
      'Content-type': 'application/json',
      'Accept': 'application/json',
    };

    var request = new Request(command: command, parameters: parameters);
    var body = json.encode(request.toJson());

    var httpResponse = await http.post(_url, body: body, headers: headers);
    if (httpResponse.statusCode == HttpStatus.ok) {
      var source = json.decode(httpResponse.body);
      var response = Response.fromJson(source);

      if (!response.isSuccess()) {
        throw new ApiException(response.code, response.error);
      }

      return response;
    } else {
      throw new ApiException(1024, "Network Error: ${httpResponse.statusCode}}");
    }
  }
}